/**
 * Name        : main.c
 * Date        : 2019
 * Description : Code to be optimised
 */

#include <cr_section_macros.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include "config_LPC1769.h"

uint32_t sum_asm(int8_t *, int len);
extern int8_t sig[2*1920];		// signal sample

void ex1()
{
	int i;
	double sum;
	uint32_t start, calc_duration_us[4]={0,0,0,0};
	int32_t sum_fx=0, sum_fx_asm;

	T0TCR=1;			// start timer 0 for time measurements

	start=T0TC;
	sum=0;
	for (i=0; i<2*1920; i++)	// calculates the sum of squares / 4
	{
		sum+=(float)sig[i]*(float)sig[i]/4.;
	}
	calc_duration_us[0]=(T0TC-start)/25;

	start=T0TC;
	// algo improved (without changing variable types), result in sum
	calc_duration_us[1]=(T0TC-start)/25;

	start=T0TC;
	// algo improved using integers, result in sum_fx
	calc_duration_us[2]=(T0TC-start)/25;

	start=T0TC;
	// algo improved using integers and assembly, result in sum_fx_asm
	calc_duration_us[3]=(T0TC-start)/25;

	printf("EX1 results: %.0f, %ld, %ld\n", sum, sum_fx, sum_fx_asm);

	for (i=0; i<4; i++)
		printf("Duration: %d [us]\n", calc_duration_us[i]);
}

#define PI 3.14159265359

void ex2()
{
	int i;
	uint32_t sum_fx=0, start, calc_duration_us[2]={0,0};
	int32_t cos_table[64];
	double sum=0;

	start=T0TC;
	for (i=0; i<2*1920; i++)	// calculates the sum of the signal elements / 4
	{
		sum+=(double)sig[i]*cos(2*PI*(float)i/64.);
	}
	calc_duration_us[0]=(T0TC-start)/25;

	// intialisation of cos_table (out of time calculation loop)
	start=T0TC;
	// optimised version with integers using cos_table[]
	calc_duration_us[1]=(T0TC-start)/25;

	// note: 1% of error is admitted on integer version
	printf("EX2 results: %.0f, %ld\n", sum, sum_fx);
	printf("Duration: %d, %d [us]\n", calc_duration_us[0], calc_duration_us[1]);
}


int main(void)
{
	ex1();
	ex2();

	return 1;
}
