/****************************************************************************
 DMA driver
****************************************************************************/

#include "LPC17xx.h"
#include "dma.h"

/* to be incremented when terminal count reached */
volatile uint32_t DMATCCount = 0;
/* to be incremented if an error occurred during transfer */
volatile uint32_t DMAErrCount = 0;

/******************************************************************************
* Descriptions:		DMA interrupt handler
******************************************************************************/
void DMA_IRQHandler (void) 
{
	/*...*/
	DMATCCount += 1;
	// Clears the channels terminal count interrupt
	LPC_GPDMA->DMACIntTCClear = 1;
}


/******************************************************************************
* Function name:  DMA_Init
*
* Description:	initialise DMA 0 channel for 32 bits access with increment on
*               source and destination addresses
*
* parameters:
*   src: source address
*   dest: destination address
*   len: number of words to transfer
*   LLI: pointer on LLI structure if used or 0
*
******************************************************************************/
void DMA_Init(uint32_t *src, uint32_t *dest, uint32_t len, uint32_t LLI)
{
	NVIC_EnableIRQ(DMA_IRQn);
	LPC_SC->PCONP |= (1 << 29);	/* Enable GPDMA clock */

	// Enabling DMA controller
	LPC_GPDMA->DMACConfig = 1;

	// Clears the channels terminal count interrupt
	LPC_GPDMA->DMACIntTCClear = 1;
	// Clears the channels error interrupt
	LPC_GPDMA->DMACIntErrClr = 1;


	LPC_GPDMACH0->DMACCSrcAddr = (uint32_t)src;
	LPC_GPDMACH0->DMACCDestAddr = (uint32_t)dest;


	LPC_GPDMACH0->DMACCLLI = LLI;

//	LPC_GPDMACH0->DMACCControl |= (len & 0xFFF); // Setting transfer size
	LPC_GPDMACH0->DMACCControl = (len | DMA_CFG);

//	LPC_GPDMACH0->DMACCControl |= (1 << 12); // Burst size of 4 for src
//	LPC_GPDMACH0->DMACCControl &= ~(3 << 13); // Setting the rest of burst src to 0
//
//	LPC_GPDMACH0->DMACCControl |= (1 << 15); // Burst size of 4 for dst
//	LPC_GPDMACH0->DMACCControl &= ~(3 << 16); // Setting the rest of burst dst to 0
//
	//  Enabling ITC (bit 15) / Enabling channel 0 (bit 1 is associated with endianness [by default val = 0] => little-endian)
	LPC_GPDMACH0->DMACCConfig = (1 << 15) | (1 << 0);
}
