/*
 *  dma_mem2lcd.h
 *
 *  Created on: July 2023
 *  Author: VP
 *  Description: general structures defining the content of the LCD commands
 */

#ifndef DMA_MEM2LCD_H_
#define DMA_MEM2LCD_H_

#include <stdint.h>

// Must be set to 1 to enable the use of the waterfall and the sun "big" sprites.
// A 0 value disable them to speed up the code transfer time on LPCExpresso
#define BIG_SPRITES 0

#define COORD_W16_SIZE 2

// defines the type of data that are sent to SSP (after the command)
typedef enum {
	COORD, 			// data coordinates are copied in SSP buffer (max DATA_MAX_SIZE bytes!)
	DATA_PTR, 		// data stored by pointer (not in SSP buffer: the data must be static!)
	DATA_RPT,		// only 2 octets of data placed in SSP buffer. The data length tells how many times they must be sent.
} data_type_t;

// specify the content of a command to be sent to the LCD which can be one of the 3 possible cases:
// - rectangle coordinates
// - image content
// - color (for a uniform colored rectangle)
typedef struct {
	uint8_t cmd;			// command byte order of LCD screen
	data_type_t cmd_type;	// command type (see description above)
	uint32_t word16_len;	// remaining number of 16 bits words to sent
	uint16_t *data_ptr;		// (big endian) data current pointer if cmd_type=DATA_PTR
	uint16_t data_rpt;		// store the color to be repeated (DATA_RPT case, big endian))
	uint16_t coord[COORD_W16_SIZE];  // store a pixel coordinates with format (min_coord_msb,
							// min_coord_lsb, max_coord_msb, max_coord_lsb, big endian)
} spi_order_t;

#endif /* DMA_MEM2LCD_H_ */
