/*
 *  dma_ssp_etu.c
 *
 *  Created on: 11 juil. 2023
 *  Author: VP
 */
#include <stdbool.h>
#include <stdint.h>
#include <assert.h>
#include "dma.h"
#include "LPC17xx.h"



void ssp0_init(uint8_t data_size_select, uint8_t scr)
{
	LPC_PINCON->PINSEL0 &= ~(3<<30);
	LPC_PINCON->PINSEL0 |=(2 << 30); // Pin 0.15 to SCK0
	LPC_PINCON->PINSEL1 &= ~(7<<2);
	LPC_PINCON->PINSEL1 |=(2 << 2); // Pin 0.17 to MISO0
	LPC_PINCON->PINSEL1 |=(2 << 4); // Pin 0.18 to MOSI0

	LPC_SC->PCONP |= (1 << 21); 	// SSP0 enable
	LPC_SC->PCLKSEL1 |= (1<<10); 	// 100MHz
	LPC_SSP0->CR0 = (scr << 8) + ((data_size_select-1) << 0);
	LPC_SSP0->CPSR = 10;			// ssp_plck/10/(scr+1) = 10 MHz / (scr+1)
	LPC_SSP0->CR1 = 1 << 1; 		// master mode +  controller enable

	LPC_SSP0->DMACR=1<<1;			// activates DMA on SSP0 side (on TX only)
}

void init_dma_ssp0()
{
	LPC_SC->PCONP |= (1 << 29);	/* Enable GPDMA clock */

	/* clear all interrupts on channel 0 */
	LPC_GPDMA->DMACIntTCClear = 0x01;
	LPC_GPDMA->DMACIntErrClr = 0x01;

	LPC_GPDMACH0->DMACCDestAddr = LPC_SSP0->DR;  //DMA_DST;

	LPC_GPDMA->DMACConfig = 0x3;	/* Enable DMA channels, big endian */

	NVIC_EnableIRQ(DMA_IRQn);			// GPDMA interrupt
	/* Enable channel and ITC bit (Interrupt at Terminal Count) and setting
	 * Transfer Type (1 << 11) to memory -> peripheral*/
	LPC_GPDMACH0->DMACCConfig = 0x8001 | (1 << 11);
}


void ssp_DMA_uint16_transfer(uint16_t *words, uint16_t word_len, bool is_incr)
{
	LPC_GPDMACH0->DMACCSrcAddr = (uint32_t)words;	//DMA_SRC;
	LPC_GPDMACH0->DMACCControl = (word_len & 0xFFF) | DMA_CFG | (is_incr << 26) | (is_incr << 27);
}

