/*
===============================================================================
 Name        : labo_dma_mem2lcd_etu.c
 Author      : VP
 Date        : July 2023
 Description : Template of lab memory to LCD by DMA
===============================================================================
*/

#include "LPC17xx.h"

#include <cr_section_macros.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>
#include "dma_mem2lcd.h"
#include "dma_ssp.h"

#define LCD_MAX_HEIGHT	(320)
#define LCD_MAX_WIDTH 	(240)
#define LCD_CS_PIN  16
#define LCD_RS_PIN  30
#define LCD_CS_ENA() {LPC_GPIO0->FIOCLR=(1<<LCD_CS_PIN);}
// set SSP0 command line to COMMAND mode:
#define SET_LCD_CMD_MODE() LPC_GPIO1->FIOCLR=(1<<30)
// set SSP0 command line to DATA mode:
#define SET_LCD_DATA_MODE() LPC_GPIO1->FIOSET=(1<<30)
// This macro can be used to ensure that there no more "physical" activity on the SSP bus.
// Note that when the TC interrupt occurs, the transfer on the bus is not terminated.
#define wait_for_ssp_bus_free() while ((LPC_SSP0->SR & SSPSR_BSY) == SSPSR_BSY);
#define SSP_BUFFER_SIZE 256

// circular buffer to store DMA orders
spi_order_t spi_order[SSP_BUFFER_SIZE];

// Send a command followed by the first frame of data
void send_dma_order(spi_order_t *tx_order)
{

}

// DMA TC interrupt.
// - the routine exits if there is no more data or command to be sent.
// - if there is still data to be sent: relaunch the DMA of the next data frame.
// - if there no more data to be sent but a new command, call send_dma_order()
//   with the corresponding command
void DMA_IRQHandler()
{

}

// Insert a new SSP command in spi_order[] pool. If there is no other order pending,
// initiate the first transfer by calling send_dma_order()
void write_spi(uint8_t cmd, data_type_t cmd_type, uint16_t *data, uint32_t byte_len)
{
	switch (cmd_type) {
		case COORD:
			break;
		case DATA_PTR:
			break;
		case DATA_RPT:
			break;
		default:
			break;
	}
}


// Define the coordinates of the rectangle to fill, where (x0, y0) is the top/left pixel
// and (x1, y1) the bottom/right pixel. Both are included in the rectangle.
void lcd_set_window(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1)
{

}

// Fill a rectangle with 'color' between (x0, y0) and (x1, y1), where (x0, y0) is the top/left pixel
// and (x1, y1) the bottom/right pixel. Both are included in the rectangle.
void fill_rectangle(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, uint16_t color)
{

}

// Displays a sprite of size width x height at position (x,y). 'data' must point on
// constant or at least non volatile data.
void display_sprite(uint16_t x, uint16_t y, uint16_t width, uint16_t height, uint16_t *data)
{

}

// LCD screen initialisation (from MyLab_lib.a)
void ILI9341_Initial();

void init_lcd_dma(void)
{
	LPC_GPIO0->FIODIR |= 1<<LCD_CS_PIN;			// activate output for LCD chip select GPIO
	LPC_GPIO1->FIODIR |= 1<<LCD_RS_PIN;			// activate output for LCD command/data GPIO
	LPC_GPIO1->FIODIR |= 1<<18;					// fix floating input line of the screen
	LPC_GPIO1->FIOPIN |= 1<<18;
	LCD_CS_ENA();								// activate LCD chip select
	ssp0_init(8, 0);							// ssp0 sck at 100/(10*(0+1))=10MHz (max freq of lcd)
	init_dma_ssp0();
	ILI9341_Initial();
}


extern unsigned short fish_width, fish_height;
extern unsigned short fish[400];
#if BIG_SPRITES
extern unsigned short waterfall_width, waterfall_height;
extern unsigned short waterfall[72000];
extern unsigned short sun_width, sun_height;
extern unsigned short sun[6059];
#endif

int main(void)
{
	init_lcd_dma();

	fill_rectangle(0, 0, 9, 9, 0xf800);		// first red rectangle for testing (to be removed)
/*	fill_rectangle(0, 0, 239, 319, 0);		// black screen
#if BIG_SPRITES
	display_sprite(0,0, waterfall_width, waterfall_height, (uint8_t *)waterfall);
#endif
	fill_rectangle(120, 300, 239, 319, 0x1f);
	display_sprite(0,300, fish_width, fish_height, (uint8_t *)fish);
	display_sprite(50,300, fish_width, fish_height, (uint8_t *)fish);
#if BIG_SPRITES
	display_sprite(0,0, sun_width, sun_height, (uint8_t *)sun);
#endif
*/
    return 0;
}
